@@ -1,8 +1,20 @@
-# #' Function to thin matrices
-# #' @param mat matrix to thin
-# #' @param thin thinning parameter
-# #' @return thinned matrix
+#' Function to get the setup from a bayesianOutput
+#' @param x bayesianOutput
+#' @return bayesianSetup
+#' @author Tankred Ott
+#' @keywords internal
+getSetup <- function(x) {
+  classes <- class(x)
+  if (any(c('mcmcSampler', 'smcSampler') %in% classes)) x$setup
+  else if (any(c('mcmcSamplerList', 'smcSamplerList') %in% classes)) x[[1]]$setup
+  else stop('Can not get setup from x')
+}
 
+#' Function to thin matrices
+#' @param mat matrix to thin
+#' @param thin thinning parameter
+#' @return thinned matrix
+#' @keywords internal
 thinMatrix <- function(mat, thin = "auto"){
   if (thin == "auto"){
     thin = max(floor(nrow(mat) / 5000),1)
@@ -16,12 +28,12 @@ thinMatrix <- function(mat, thin = "auto"){
 }
 
 
-# #' Function to scale matrices
-# #' @param mat matrix to scale
-# #' @param min minimum value
-# #' @param max maximum value
-# #' @return sclaed matrix
-
+#' Function to scale matrices
+#' @param mat matrix to scale
+#' @param min minimum value
+#' @param max maximum value
+#' @return sclaed matrix
+#' @keywords internal
 scaleMatrix <- function(mat, min, max){
   if(class(mat) %in% c("matrix", "data.frame")){
     for(i in 1:ncol(mat)){
@@ -39,7 +51,7 @@ scaleMatrix <- function(mat, min, max){
 #' @param LP2 log posterior old position
 #' @param LP1 log posterior of proposal
 #' @param tempering value for tempering
-#' @export
+#' @keywords internal
 metropolisRatio <- function(LP2, LP1, tempering = 1){
   # this catches two -Inf cases / I wonder if we should throw a warning in this case
   if( is.na(LP2 - LP1)) out = -Inf
@@ -51,7 +63,7 @@ metropolisRatio <- function(LP2, LP1, tempering = 1){
 #' Calculates the panel combination for par(mfrow = )
 #' @author Florian Hartig
 #' @param x the desired number of panels 
-# #' @export
+#' @keywords internal
 getPanels <- function(x){
   if (x <= 0) stop("number can't be < 1")
   
@@ -70,7 +82,7 @@ getPanels <- function(x){
 #' @param x matrix or vector
 #' @param numSamples number of samples (rows) to be drawn
 #' @details Gets n equally spaced samples (rows) from a matrix and returns a new matrix (or vector) containing those samples
-# #' @export
+#' @keywords internal
 sampleEquallySpaced <- function(x, numSamples) {
   # wrong input: x is neither vector nor matrix
   if (!is.matrix(x) && !is.vector(x)) {
@@ -124,6 +136,7 @@ sampleEquallySpaced <- function(x, numSamples) {
 #' @details Checks if the thin argument is consistent with the data consisting of nTotalSamples samples/rows and corrects thin if not.
 #' @author Tankred Ott
 # #' @export
+#' @keywords internal
 correctThin <- function(nTotalSamples, thin, autoThinFraction = 0.001) {
   if (autoThinFraction > 1 || autoThinFraction <= 0) {
     stop("autoThinFraction must be greater than 0 and less than 1!")
@@ -147,6 +160,7 @@ correctThin <- function(nTotalSamples, thin, autoThinFraction = 0.001) {
 #' @param x Vector of values
 #' @param from vector, interval of which x are elements. from[1] must be the lower, from[2] the upper bound.
 #' @param to vector, interval to which the elements should be scaled. to[1] must be the lower, to[2] the upper bound.
+#' @keywords internal
 rescale <- function (x, from, to) {
   # scale x from 0 to 1
   x <- (x - from[1]) / (from[2] - from[1])
